'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.arrayDiff = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = configureTestStore;

var _redux = require('redux');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Helper function to aid in async dispatch testing.
 * @param {Store} store The redux store
 * @param {Function} stateFunc When called, returns the part of the state we care about
 * @param {Function} actionListFunc When called, returns the list of actions
 * @param {Array} blacklistedActionTypes A list of action types which are ignored
 * @returns {Function} Returns a function which will dispatch an action and take a callback
 * to be executed after the store updates its state. Looks like:
 *   (action, expectedActions) => Promise
 * where
 *   action is an action to be dispatched (or falsey if no action)
 *   expectedActions is a list of actions to listen for
 *   The returned Promise is executed after the number of actions in expectedCalls
 *   is met
 */
function createDispatchThen(store, stateFunc, actionListFunc, blacklistedActionTypes) {
  var listenForActions = createListenForActions(store, stateFunc, actionListFunc, blacklistedActionTypes);
  return function (action, actionTypes) {
    if ((typeof actionTypes === 'undefined' ? 'undefined' : _typeof(actionTypes)) !== 'object') {
      throw new Error("Expected a list of actions for second parameter of dispatchThen");
    }
    return listenForActions(actionTypes, function () {
      store.dispatch(action);
    });
  };
}

/**
 * Listen for actions. After some actions have been processed execute
 * a callback so the user can verify state.
 *
 * @param {Store} store The redux store
 * @param {Function} stateFunc When called, returns the part of the state we care about
 * @param {Function} actionListFunc When called, returns the list of actions
 * @param {Array} blacklistedActionTypes A list of action types which are ignored
 * @returns {Function} Returns a function:
 *   (expectedActionTypes, callback) => Promise
 * where
 *   expectedActionTypes is a list of actions that must be the only actions dispatched
 *     by the code in the callback,
 *   callback which runs the code that will dispatch actions indirectly
 * and returns a Promise that resolves after the expected actions have occurred.
 *
 * Example:
 *   const listenForActions = store.createListenForActions(state => state);
 *   listenForActions(['UPDATE_SELECTED_CHAPTERS', 'UPDATE_SEAT_COUNT'], () => {
 *     runFuncWhichDispatchesTwoActions();
 *   }).then(state => {
 *     assert.deepEqual(state, {
 *       ...
 *     });
 *   });
 */
function createListenForActions(store, stateFunc, actionListFunc, blacklistedActionTypes) {
  var unsubscribe = void 0;
  var actionTypesFunc = function actionTypesFunc() {
    return actionListFunc().map(function (action) {
      return action.type;
    }).sort();
  };

  return function (actionTypes, callback) {
    if ((typeof actionTypes === 'undefined' ? 'undefined' : _typeof(actionTypes)) !== 'object') {
      throw new Error("Expected a list of actions for first parameter of listenForActions");
    }
    if (typeof callback !== 'function') {
      throw new Error("Callback argument is required for listenForActions");
    }
    return new Promise(function (resolve, reject) {
      // Note: we are not using Sets because we care about preserving duplicate values
      var expectedActionTypes = Array.from(actionTypes).concat(actionTypesFunc()).sort().filter(function (type) {
        return blacklistedActionTypes.indexOf(type) === -1;
      });

      // If we called this twice unsubscribe the old instance from the store
      if (unsubscribe !== undefined) {
        unsubscribe();
      }
      if (_lodash2.default.isEqual(actionTypesFunc(), expectedActionTypes)) {
        // No actions expected, run callback now
        callback();

        resolve(stateFunc(store.getState()));
      } else {
        var callbackRan = false;

        var resolver = function resolver() {
          // Get current action list
          var actionListTypes = actionTypesFunc();
          if (_lodash2.default.isEqual(actionListTypes, expectedActionTypes)) {
            resolve(stateFunc(store.getState()));
          } else if (actionListTypes.length > expectedActionTypes.length) {
            reject(new Error(formatActionErrors(expectedActionTypes, actionListTypes)));
          }
        };

        unsubscribe = store.subscribe(function () {
          if (callbackRan) {
            // wait until after callback() completes to resolve so we can detect thrown errors
            // and actions which shouldn't have been dispatched
            resolver();
          }
        });

        callback();
        callbackRan = true;
        // If already resolved, this should get ignored
        resolver();
      }
    });
  };
}

// Keep a list of actions executed for testing purposes
var subscriber = function subscriber(subscribe) {
  return function (store) {
    return function (next) {
      return function (action) {
        if (subscribe !== undefined) {
          subscribe(action);
        }
        return next(action);
      };
    };
  };
};

function configureTestStore(rootReducer, initialState) {
  var blacklistedActionTypes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var actionList = [];

  var createStoreWithMiddleware = (0, _redux.applyMiddleware)(_reduxThunk2.default, subscriber(function (action) {
    actionList = actionList.concat(action);
  }))(_redux.createStore);

  var actionListFunc = function actionListFunc() {
    return actionList.filter(function (action) {
      return blacklistedActionTypes.indexOf(action.type) === -1;
    });
  };

  var store = createStoreWithMiddleware(rootReducer, initialState);
  var identity = function identity(state) {
    return state;
  };
  store.createDispatchThen = function () {
    var stateFunc = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : identity;
    return createDispatchThen(store, stateFunc, actionListFunc, blacklistedActionTypes);
  };
  store.createListenForActions = function () {
    var stateFunc = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : identity;
    return createListenForActions(store, stateFunc, actionListFunc, blacklistedActionTypes);
  };
  return store;
}

// returns xs - ys
var arrayDiff = exports.arrayDiff = function arrayDiff(xs, ys) {
  var _ys = ys.concat();

  return xs.map(function (x) {
    var index = _ys.indexOf(x);
    if (index < 0) {
      return x;
    } else {
      _ys.splice(index, 1);
    }
  }).filter(function (x) {
    return x !== undefined;
  });
};

var formatActions = function formatActions(xs) {
  return xs.map(function (x) {
    return '    ' + x + ',';
  });
};

var formatActionErrors = function formatActionErrors(expected, received) {
  var lines = ["ReduxAsserts: didn't receive the expected actions\n"];

  var unexpected = arrayDiff(received, expected);
  if (unexpected.length !== 0) {
    lines = lines.concat("Unexpected actions:", "[", formatActions(unexpected), "]", "\n");
  }

  var unreceived = arrayDiff(expected, received);
  if (unreceived.length !== 0) {
    lines = lines.concat("Expected, unreceived actions:", "[", formatActions(unreceived), "]", "\n");
  }
  return lines.join('\n');
};